@@ -1,5 +1,5 @@
 /************************************************************************************
-   Copyright (C) 2020, 2023 MariaDB Corporation AB
+   Copyright (C) 2020, 2024 MariaDB Corporation plc
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -31,7 +31,6 @@
 #include "ColumnDefinitionCapi.h"
 #include "ExceptionFactory.h"
 #include "SqlStates.h"
-//#include "com/Packet.h"
 #include "com/RowProtocol.h"
 #include "protocol/capi/BinRowProtocolCapi.h"
 #include "protocol/capi/TextRowProtocolCapi.h"
@@ -43,78 +42,22 @@ namespace mariadb
 {
 namespace capi
 {
-  /**
-    * Create Streaming resultSet.
-    *
-    * @param results results
-    * @param protocol current protocol
-    * @param spr ServerPrepareResult
-    * @param callableResult is it from a callableStatement ?
-    * @param eofDeprecated is EOF deprecated
-    */
-  SelectResultSetCapi::SelectResultSetCapi(Results* results,
-                                           Protocol* protocol,
-                                           ServerPrepareResult* spr,
-                                           bool callableResult,
-                                           bool eofDeprecated)
-    :
-      options(protocol->getOptions()),
-      columnsInformation(spr->getColumns()),
-      columnInformationLength(static_cast<int32_t>(columnsInformation.size())),
-      noBackslashEscapes(protocol->noBackslashEscapes()),
-      protocol(protocol),
-      callableResult(callableResult),
-      statement(results->getStatement()),
-      capiConnHandle(nullptr),
-      capiStmtHandle(spr->getStatementId()),
-      dataSize(0),
-      fetchSize(results->getFetchSize()),
-      resultSetScrollType(results->getResultSetScrollType()),
-      columnNameMap(columnsInformation),
-      eofDeprecated(eofDeprecated),
-      forceAlias(false)
-  {
-    if (fetchSize == 0 || callableResult) {
-      data.reserve(10);
-      if (mysql_stmt_store_result(capiStmtHandle)) {
-        throwStmtError(capiStmtHandle);
-      }
-      dataSize= static_cast<std::size_t>(mysql_stmt_num_rows(capiStmtHandle));
-      streaming= false;
-      resetVariables();
-    }
-    else {
-      lock= protocol->getLock();
-
-      protocol->setActiveStreamingResult(results);
-
-      protocol->removeHasMoreResults();
-      data.reserve(std::max(10, fetchSize)); // Same
-      nextStreamingValue();
-      streaming= true;
-    }
-
-    row.reset(new capi::BinRowProtocolCapi(columnsInformation, columnInformationLength, results->getMaxFieldSize(), options, capiStmtHandle));
-  }
-
-
   SelectResultSetCapi::SelectResultSetCapi(Results * results,
                                            Protocol * _protocol,
                                            MYSQL* capiConnHandle,
                                            bool eofDeprecated)
-    :
+    : SelectResultSet(results->getFetchSize()),
       options(_protocol->getOptions()),
       noBackslashEscapes(_protocol->noBackslashEscapes()),
       protocol(_protocol),
       callableResult(false),
       statement(results->getStatement()),
       capiConnHandle(capiConnHandle),
-      capiStmtHandle(nullptr),
       dataSize(0),
-      fetchSize(results->getFetchSize()),
       resultSetScrollType(results->getResultSetScrollType()),
       columnNameMap(columnsInformation),
       eofDeprecated(eofDeprecated),
+      lock(protocol->getLock()),
       forceAlias(false)
   {
     MYSQL_RES* textNativeResults= nullptr;
@@ -130,7 +73,6 @@ namespace capi
       resetVariables();
     }
     else {
-      lock= protocol->getLock();
       protocol->setActiveStreamingResult(results);
 
       protocol->removeHasMoreResults();
@@ -171,7 +113,7 @@ namespace capi
     std::vector<std::vector<sql::bytes>>& resultSet,
     Protocol* _protocol,
     int32_t resultSetScrollType)
-    :
+    : SelectResultSet(0),
       columnsInformation(columnInformation),
       columnInformationLength(static_cast<int32_t>(columnInformation.size())),
       noBackslashEscapes(false),
@@ -181,15 +123,12 @@ namespace capi
       statement(nullptr),
       row(new capi::TextRowProtocolCapi(0, this->options, nullptr)),
       capiConnHandle(nullptr),
-      capiStmtHandle(nullptr),
-      streaming(false),
       data(std::move(resultSet)),
       dataSize(data.size()),
-      fetchSize(0),
       resultSetScrollType(resultSetScrollType),
-      rowPointer(-1),
       columnNameMap(columnsInformation),
       eofDeprecated(false),
+      lock(nullptr),
       forceAlias(false)
   {
     if (protocol != nullptr) {
@@ -213,7 +152,7 @@ namespace capi
     * @return true if streaming is finished
     */
   bool SelectResultSetCapi::isFullyLoaded() const {
-
+    // result-set is fully loaded when reaching EOF packet.
     return isEof;
   }
 
@@ -227,11 +166,7 @@ namespace capi
 
   const char * SelectResultSetCapi::getErrMessage()
   {
-    if (capiStmtHandle != nullptr)
-    {
-      return mysql_stmt_error(capiStmtHandle);
-    }
-    else if (capiConnHandle != nullptr)
+    if (capiConnHandle != nullptr)
     {
       return mysql_error(capiConnHandle);
     }
@@ -241,11 +176,7 @@ namespace capi
 
   const char * SelectResultSetCapi::getSqlState()
   {
-    if (capiStmtHandle != nullptr)
-    {
-      return mysql_stmt_error(capiStmtHandle);
-    }
-    else if (capiConnHandle != nullptr)
+    if (capiConnHandle != nullptr)
     {
       return mysql_error(capiConnHandle);
     }
@@ -254,11 +185,7 @@ namespace capi
 
   uint32_t SelectResultSetCapi::getErrNo()
   {
-    if (capiStmtHandle != nullptr)
-    {
-      return mysql_stmt_errno(capiStmtHandle);
-    }
-    else if (capiConnHandle != nullptr)
+    if (capiConnHandle != nullptr)
     {
       return mysql_errno(capiConnHandle);
     }
@@ -267,11 +194,7 @@ namespace capi
 
   uint32_t SelectResultSetCapi::warningCount()
   {
-    if (capiStmtHandle != nullptr)
-    {
-      return mysql_stmt_warning_count(capiStmtHandle);
-    }
-    else if (capiConnHandle != nullptr)
+    if (capiConnHandle != nullptr)
     {
       return mysql_warning_count(capiConnHandle);
     }
@@ -298,18 +221,42 @@ namespace capi
 
   /**
     * When protocol has a current Streaming result (this) fetch all to permit another query is
-    * executing.
+    * executing. The lock should be acquired before calling this method
     *
     * @throws SQLException if any error occur
     */
   void SelectResultSetCapi::fetchRemaining() {
     if (!isEof) {
-      std::lock_guard<std::mutex> localScopeLock(*lock);
-      fetchRemainingInternal();
+      try {
+        lastRowPointer= -1;
+        // Making copy of the current row in C/C in local cache.
+        if (!isEof && dataSize > 0 && fetchSize == 1) {
+          // We need to grow the array till current size. Its main purpose is to create room for newly fetched
+          // fetched row, so it grows till dataSize + 1. But we need to space for already fetched(from server)
+          // row. Thus fooling growDataArray by decrementing dataSize
+          --dataSize;
+          growDataArray();
+          // Since index of the last row is smaller from dataSize by 1, we have correct index
+          row->cacheCurrentRow(data[dataSize], columnsInformation.size());
+          rowPointer= 0;
+          resetRow();
+          ++dataSize;
+        }
+        while (!isEof) {
+          addStreamingValue(true);
+        }
+      }
+      catch (SQLException& queryException) {
+        ExceptionFactory::INSTANCE.create(queryException).Throw();
+      }
+      catch (std::exception& ioe) {
+        handleIoException(ioe);
+      }
+      ++dataFetchTime;
     }
   }
 
-  void SelectResultSetCapi::handleIoException(std::exception& ioe)
+  void SelectResultSetCapi::handleIoException(std::exception& ioe) const
   {
     ExceptionFactory::INSTANCE.create(
         "Server has closed the connection. \n"
@@ -334,25 +281,9 @@ namespace capi
       dataSize= 0;
     }
 
-    addStreamingValue();
+    addStreamingValue(fetchSize > 1);
   }
 
-  /**
-    * This permit to add next streaming values to existing resultSet.
-    *
-    * @throws IOException if socket exception occur
-    * @throws SQLException if server return an unexpected error
-    */
-  void SelectResultSetCapi::addStreamingValue() {
-
-    int32_t fetchSizeTmp= fetchSize;
-    while (fetchSizeTmp > 0 && readNextValue()) {
-      fetchSizeTmp--;
-    }
-    ++dataFetchTime;
-  }
-
-
   /**
     * Read next value.
     *
@@ -360,10 +291,9 @@ namespace capi
     * @throws IOException exception
     * @throws SQLException exception
     */
-  bool SelectResultSetCapi::readNextValue()
+  bool SelectResultSetCapi::readNextValue(bool cacheLocally)
   {
     switch (row->fetchNext()) {
-
     case MYSQL_DATA_TRUNCATED: {
       /*protocol->removeActiveStreamingResult();
       protocol->removeHasMoreResults();*/
@@ -378,7 +308,13 @@ namespace capi
         nullptr,
         false);*/
     }
-
+    case 1: {
+      // mysql_fetch_row may return NULL if there are no more rows, but also possible an error, that in case of result use, detected on first fetch
+      if (capiConnHandle != nullptr && mysql_errno(capiConnHandle) != 0) {
+        throw SQLException(mysql_error(capiConnHandle), mysql_sqlstate(capiConnHandle), mysql_errno(capiConnHandle));
+      }
+      // else we are falling thru to MYSQL_NO_DATA
+    }
     case MYSQL_NO_DATA: {
       uint32_t serverStatus;
       if (protocol) {
@@ -413,16 +349,19 @@ namespace capi
           protocol->removeActiveStreamingResult();
         }
       }
-
       resetVariables();
       return false;
     }
     }
 
-    if (dataSize + 1 >= data.size()) {
-      growDataArray();
+    if (cacheLocally) {
+      if (dataSize + 1 >= data.size()) {
+        growDataArray();
+      }
+      row->cacheCurrentRow(data[dataSize], columnsInformation.size());
     }
     ++dataSize;
+
     return true;
   }
 
@@ -461,12 +400,12 @@ namespace capi
   }
 
   void SelectResultSetCapi::addRowData(std::vector<sql::bytes>& rawData) {
-    if (dataSize +1 >= data.size()) {
+    if (dataSize + 1 >= data.size()) {
       growDataArray();
     }
     data[dataSize]= rawData;
     rowPointer= static_cast<int32_t>(dataSize);
-    dataSize++;
+    ++dataSize;
   }
 
   /*int32_t SelectResultSetCapi::skipLengthEncodedValue(std::string& buf, int32_t pos) {
@@ -544,35 +483,10 @@ namespace capi
 
   /** Close resultSet. */
   void SelectResultSetCapi::close() {
-    isClosedFlag= true;
-    if (!isEof) {
-      std::unique_lock<std::mutex> localScopeLock(*lock);
-      try {
-        while (!isEof) {
-          dataSize= 0; // to avoid storing data
-          readNextValue();
-        }
-      }
-      catch (SQLException& queryException) {
-        ExceptionFactory::INSTANCE.create(queryException).Throw();
-      }
-      catch (std::runtime_error& ioe) {
-        resetVariables();
-        localScopeLock.unlock();
-        handleIoException(ioe);
-      }
-    }
-    checkOut();
-    resetVariables();
-
-    data.clear();
-
-    if (statement != nullptr) {
-      statement->checkCloseOnCompletion(this);
-      statement= nullptr;
-    }
+    realClose(false);
   }
 
+
   void SelectResultSetCapi::resetVariables() {
     protocol= nullptr;
     isEof= true;
@@ -626,29 +540,30 @@ namespace capi
         }
       }
 
-
       rowPointer= static_cast<int32_t>(dataSize);
       return false;
     }
   }
 
-
-  void SelectResultSetCapi::resetRow()
+  // It has to be const, because it's called by getters, and properties it changes are mutable
+  void SelectResultSetCapi::resetRow() const
   {
     if (data.size() > 0) {
-      row->resetRow(data[rowPointer]);
+      row->resetRow(const_cast<std::vector<sql::bytes>&>(data[rowPointer]));
     }
     else {
       if (rowPointer != lastRowPointer + 1) {
         row->installCursorAtPosition(rowPointer);
       }
-      row->fetchNext();
+      if (!streaming) {
+        row->fetchNext();
+      }
     }
     lastRowPointer= rowPointer;
   }
 
 
-  void SelectResultSetCapi::checkObjectRange(int32_t position) {
+  void SelectResultSetCapi::checkObjectRange(int32_t position) const {
     if (rowPointer < 0) {
       throw SQLDataException("Current position is before the first row", "22023");
     }
@@ -667,6 +582,7 @@ namespace capi
     row->setPosition(position - 1);
   }
 
+
   SQLWarning* SelectResultSetCapi::getWarnings() {
     if (this->statement == nullptr) {
       return nullptr;
@@ -680,7 +596,7 @@ namespace capi
     }
   }
 
-  bool SelectResultSetCapi::isBeforeFirst() {
+  bool SelectResultSetCapi::isBeforeFirst() const {
     checkClose();
     return (dataFetchTime >0) ? rowPointer == -1 && dataSize > 0 : rowPointer == -1;
   }
@@ -714,11 +630,11 @@ namespace capi
       // has read all data and pointer is after last result
       // so result would have to always to be true,
       // but when result contain no row at all jdbc say that must return false
-      return dataSize >0 ||dataFetchTime >1;
+      return dataSize > 0 || dataFetchTime > 1;
     }
   }
 
-  bool SelectResultSetCapi::isFirst() {
+  bool SelectResultSetCapi::isFirst() const {
     checkClose();
     return /*dataFetchTime == 1 && */rowPointer == 0 && dataSize > 0;
   }
@@ -763,7 +679,11 @@ namespace capi
 
   void SelectResultSetCapi::afterLast() {
     checkClose();
-    fetchRemaining();
+    if (!isEof) {
+      //SelectResultSet objects only have lock if streaming
+      std::lock_guard<std::mutex> localScopeLock(*lock);
+      fetchRemaining();
+    }
     rowPointer= static_cast<int32_t>(dataSize);
   }
 
@@ -780,7 +700,11 @@ namespace capi
 
   bool SelectResultSetCapi::last() {
     checkClose();
-    fetchRemaining();
+    if (!isEof) {
+      //SelectResultSet objects only have lock if streaming
+      std::lock_guard<std::mutex> localScopeLock(*lock);
+      fetchRemaining();
+    }
     rowPointer= static_cast<int32_t>(dataSize) - 1;
     return dataSize > 0;
   }
@@ -804,8 +728,11 @@ namespace capi
       rowPointer= rowPos - 1;
       return true;
     }
-
-    fetchRemaining();
+    if (!isEof) {
+      //SelectResultSet objects only have lock if streaming
+      std::lock_guard<std::mutex> localScopeLock(*lock);
+      fetchRemaining();
+    }
 
     if (rowPos >= 0) {
 
@@ -813,20 +740,16 @@ namespace capi
         rowPointer= rowPos - 1;
         return true;
       }
-
       rowPointer= static_cast<int32_t>(dataSize);
       return false;
-
     }
     else {
 
       // Need to cast, or otherwise the result would be size_t -> always not negative
       if (static_cast<int64_t>(dataSize) + rowPos >= 0) {
-
         rowPointer= static_cast<int32_t>(dataSize + rowPos);
         return true;
       }
-
       rowPointer= -1;
       return false;
     }
@@ -865,7 +788,7 @@ namespace capi
     return false;
   }
 
-  int32_t SelectResultSetCapi::getFetchDirection() {
+  int32_t SelectResultSetCapi::getFetchDirection() const {
     return FETCH_UNKNOWN;
   }
 
@@ -876,7 +799,7 @@ namespace capi
     }
   }
 
-  int32_t SelectResultSetCapi::getFetchSize() {
+  int32_t SelectResultSetCapi::getFetchSize() const {
     return this->fetchSize;
   }
 
@@ -897,21 +820,21 @@ namespace capi
     this->fetchSize= fetchSize;
   }
 
-  int32_t SelectResultSetCapi::getType() {
+  int32_t SelectResultSetCapi::getType()  const {
     return resultSetScrollType;
   }
 
-  int32_t SelectResultSetCapi::getConcurrency() {
+  int32_t SelectResultSetCapi::getConcurrency() const {
     return CONCUR_READ_ONLY;
   }
 
-  void SelectResultSetCapi::checkClose() {
+  void SelectResultSetCapi::checkClose() const {
     if (isClosedFlag) {
       throw SQLException("Operation not permit on a closed resultSet", "HY000");
     }
   }
 
-  bool SelectResultSetCapi::isCallableResult() {
+  bool SelectResultSetCapi::isCallableResult() const {
     return callableResult;
   }
 
@@ -929,17 +852,17 @@ namespace capi
   }
 
   /** {inheritDoc}. */
-  bool SelectResultSetCapi::wasNull() {
+  bool SelectResultSetCapi::wasNull() const {
     return row->wasNull();
   }
 
-  bool SelectResultSetCapi::isNull(int32_t columnIndex)
+  bool SelectResultSetCapi::isNull(int32_t columnIndex) const
   {
     checkObjectRange(columnIndex);
     return row->lastValueWasNull();
   }
 
-  bool SelectResultSetCapi::isNull(const SQLString & columnLabel)
+  bool SelectResultSetCapi::isNull(const SQLString & columnLabel) const
   {
     return isNull(findColumn(columnLabel));
   }
@@ -962,21 +885,14 @@ namespace capi
 #endif
 
   /** {inheritDoc}. */
-  SQLString SelectResultSetCapi::getString(int32_t columnIndex)
+  SQLString SelectResultSetCapi::getString(int32_t columnIndex) const
   {
     checkObjectRange(columnIndex);
-    std::unique_ptr<SQLString> res= row->getInternalString(columnsInformation[columnIndex -1].get());
-
-    if (res) {
-      return std::move(*res);
-    }
-    else {
-      return emptyStr;
-    }
+    return std::move(row->getInternalString(columnsInformation[columnIndex - 1].get()));
   }
 
   /** {inheritDoc}. */
-  SQLString SelectResultSetCapi::getString(const SQLString& columnLabel) {
+  SQLString SelectResultSetCapi::getString(const SQLString& columnLabel) const {
     return getString(findColumn(columnLabel));
   }
 
@@ -995,7 +911,7 @@ namespace capi
   }
 
   /** {inheritDoc}. */
-  std::istream* SelectResultSetCapi::getBinaryStream(int32_t columnIndex) {
+  std::istream* SelectResultSetCapi::getBinaryStream(int32_t columnIndex) const {
     checkObjectRange(columnIndex);
     if (row->lastValueWasNull()) {
       return nullptr;
@@ -1005,53 +921,49 @@ namespace capi
   }
 
   /** {inheritDoc}. */
-  std::istream* SelectResultSetCapi::getBinaryStream(const SQLString& columnLabel) {
+  std::istream* SelectResultSetCapi::getBinaryStream(const SQLString& columnLabel) const {
     return getBinaryStream(findColumn(columnLabel));
   }
 
   /** {inheritDoc}. */
-  int32_t SelectResultSetCapi::getInt(int32_t columnIndex) {
+  int32_t SelectResultSetCapi::getInt(int32_t columnIndex) const {
     checkObjectRange(columnIndex);
     return row->getInternalInt(columnsInformation[columnIndex -1].get());
   }
 
-  /** {inheritDoc}. */  int32_t SelectResultSetCapi::getInt(const SQLString& columnLabel) {
+  /** {inheritDoc}. */  int32_t SelectResultSetCapi::getInt(const SQLString& columnLabel) const {
     return getInt(findColumn(columnLabel));
   }
 
   /** {inheritDoc}. */
-  int64_t SelectResultSetCapi::getLong(const SQLString& columnLabel) {
+  int64_t SelectResultSetCapi::getLong(const SQLString& columnLabel) const {
     return getLong(findColumn(columnLabel));
   }
 
   /** {inheritDoc}. */
-  int64_t SelectResultSetCapi::getLong(int32_t columnIndex) {
+  int64_t SelectResultSetCapi::getLong(int32_t columnIndex) const {
     checkObjectRange(columnIndex);
     return row->getInternalLong(columnsInformation[columnIndex -1].get());
   }
 
 
-  uint64_t SelectResultSetCapi::getUInt64(const SQLString & columnLabel)
-  {
+  uint64_t SelectResultSetCapi::getUInt64(const SQLString & columnLabel) const {
     return getUInt64(findColumn(columnLabel));
   }
 
 
-  uint64_t SelectResultSetCapi::getUInt64(int32_t columnIndex)
-  {
+  uint64_t SelectResultSetCapi::getUInt64(int32_t columnIndex) const {
     checkObjectRange(columnIndex);
     return static_cast<uint64_t>(row->getInternalULong(columnsInformation[columnIndex -1].get()));
   }
 
 
-  uint32_t SelectResultSetCapi::getUInt(const SQLString& columnLabel)
-  {
+  uint32_t SelectResultSetCapi::getUInt(const SQLString& columnLabel) const {
     return getUInt(findColumn(columnLabel));
   }
 
 
-  uint32_t SelectResultSetCapi::getUInt(int32_t columnIndex)
-  {
+  uint32_t SelectResultSetCapi::getUInt(int32_t columnIndex) const {
     checkObjectRange(columnIndex);
 
     ColumnDefinition* columnInfo= columnsInformation[columnIndex - 1].get();
@@ -1064,23 +976,23 @@ namespace capi
 
 
   /** {inheritDoc}. */
-  float SelectResultSetCapi::getFloat(const SQLString& columnLabel) {
+  float SelectResultSetCapi::getFloat(const SQLString& columnLabel) const {
     return getFloat(findColumn(columnLabel));
   }
 
   /** {inheritDoc}. */
-  float SelectResultSetCapi::getFloat(int32_t columnIndex) {
+  float SelectResultSetCapi::getFloat(int32_t columnIndex) const {
     checkObjectRange(columnIndex);
     return row->getInternalFloat(columnsInformation[columnIndex -1].get());
   }
 
   /** {inheritDoc}. */
-  long double SelectResultSetCapi::getDouble(const SQLString& columnLabel) {
+  long double SelectResultSetCapi::getDouble(const SQLString& columnLabel) const {
     return getDouble(findColumn(columnLabel));
   }
 
   /** {inheritDoc}. */
-  long double SelectResultSetCapi::getDouble(int32_t columnIndex) {
+  long double SelectResultSetCapi::getDouble(int32_t columnIndex) const {
     checkObjectRange(columnIndex);
     return row->getInternalDouble(columnsInformation[columnIndex -1].get());
   }
@@ -1199,12 +1111,12 @@ namespace capi
   }
 
   /** {inheritDoc}. */
-  sql::ResultSetMetaData* SelectResultSetCapi::getMetaData() {
+  sql::ResultSetMetaData* SelectResultSetCapi::getMetaData() const {
     return new MariaDbResultSetMetaData(columnsInformation, options, forceAlias);
   }
 
   /** {inheritDoc}. */
-  int32_t SelectResultSetCapi::findColumn(const SQLString& columnLabel) {
+  int32_t SelectResultSetCapi::findColumn(const SQLString& columnLabel) const {
     return columnNameMap.getIndex(columnLabel) + 1;
   }
 
@@ -1489,55 +1401,55 @@ namespace capi
 #endif
 
   /** {inheritDoc}. */
-  Blob* SelectResultSetCapi::getBlob(int32_t columnIndex) {
+  Blob* SelectResultSetCapi::getBlob(int32_t columnIndex) const {
     return getBinaryStream(columnIndex);
   }
 
   /** {inheritDoc}. */
-  Blob* SelectResultSetCapi::getBlob(const SQLString& columnLabel) {
+  Blob* SelectResultSetCapi::getBlob(const SQLString& columnLabel) const {
     return getBlob(findColumn(columnLabel));
   }
 
   /** {inheritDoc}. */
-  RowId* SelectResultSetCapi::getRowId(int32_t /*columnIndex*/) {
+  RowId* SelectResultSetCapi::getRowId(int32_t /*columnIndex*/) const {
     throw ExceptionFactory::INSTANCE.notSupported("RowIDs not supported");
   }
 
   /** {inheritDoc}. */
-  RowId* SelectResultSetCapi::getRowId(const SQLString& /*columnLabel*/) {
+  RowId* SelectResultSetCapi::getRowId(const SQLString& /*columnLabel*/) const {
     throw ExceptionFactory::INSTANCE.notSupported("RowIDs not supported");
   }
 
   /** {inheritDoc}. */
-  bool SelectResultSetCapi::getBoolean(int32_t index) {
+  bool SelectResultSetCapi::getBoolean(int32_t index) const {
     checkObjectRange(index);
     return row->getInternalBoolean(columnsInformation[static_cast<std::size_t>(index) -1].get());
   }
 
   /** {inheritDoc}. */
-  bool SelectResultSetCapi::getBoolean(const SQLString& columnLabel) {
+  bool SelectResultSetCapi::getBoolean(const SQLString& columnLabel) const {
     return getBoolean(findColumn(columnLabel));
   }
 
   /** {inheritDoc}. */
-  int8_t SelectResultSetCapi::getByte(int32_t index) {
+  int8_t SelectResultSetCapi::getByte(int32_t index) const {
     checkObjectRange(index);
     return row->getInternalByte(columnsInformation[static_cast<std::size_t>(index) - 1].get());
   }
 
   /** {inheritDoc}. */
-  int8_t SelectResultSetCapi::getByte(const SQLString& columnLabel) {
+  int8_t SelectResultSetCapi::getByte(const SQLString& columnLabel) const {
     return getByte(findColumn(columnLabel));
   }
 
   /** {inheritDoc}. */
-  short SelectResultSetCapi::getShort(int32_t index) {
+  short SelectResultSetCapi::getShort(int32_t index) const {
     checkObjectRange(index);
     return row->getInternalShort(columnsInformation[static_cast<std::size_t>(index) - 1].get());
   }
 
   /** {inheritDoc}. */
-  short SelectResultSetCapi::getShort(const SQLString& columnLabel) {
+  short SelectResultSetCapi::getShort(const SQLString& columnLabel) const {
     return getShort(findColumn(columnLabel));
   }
 
@@ -1590,7 +1502,7 @@ namespace capi
     throw ExceptionFactory::INSTANCE.notSupported(NOT_UPDATABLE_ERROR);
   }
 
-  std::size_t sql::mariadb::capi::SelectResultSetCapi::rowsCount()
+  std::size_t sql::mariadb::capi::SelectResultSetCapi::rowsCount() const
   {
     return dataSize;
   }
@@ -1955,7 +1867,7 @@ namespace capi
 #endif
 
   /** {inheritDoc}. */
-  int32_t SelectResultSetCapi::getHoldability() {
+  int32_t SelectResultSetCapi::getHoldability() const {
     return ResultSet::HOLD_CURSORS_OVER_COMMIT;
   }
 
@@ -2007,47 +1919,64 @@ namespace capi
     return dataSize;
   }
 
+
   bool SelectResultSetCapi::isBinaryEncoded() {
     return row->isBinaryEncoded();
   }
 
 
+  void SelectResultSetCapi::realClose(bool noLock)
+  {
+    isClosedFlag= true;
+    if (!isEof) {
+      if (!noLock) {
+        lock->lock();
+      }
+      try {
+        while (!isEof) {
+          dataSize = 0; // to avoid storing data
+          readNextValue();
+        }
+      }
+      catch (SQLException & queryException) {
+        if (!noLock) {
+          lock->unlock();
+        }
+        ExceptionFactory::INSTANCE.create(queryException).Throw();
+      }
+      catch (std::runtime_error & ioe) {
+        if (!noLock) {
+          lock->unlock();
+        }
+        resetVariables();
+        handleIoException(ioe);
+      }
+      if (!noLock) {
+        lock->unlock();
+      }
+    }
+
+    checkOut();
+
+    resetVariables();
+
+    data.clear();
+
+    if (statement != nullptr) {
+      statement->checkCloseOnCompletion(this);
+      statement = nullptr;
+    }
+  }
+
+
   void SelectResultSetCapi::cacheCompleteLocally() {
 
     if (fetchSize > 0) {
       fetchRemaining();
     }
-    else if (row->isBinaryEncoded()) {
-      if (data.size()) {
-        // we have already it cached
-        return;
-      }
-      auto preservedPosition= rowPointer;
-      // fetchRemaining does remaining stream
-      if (streaming) {
-        fetchRemainingInternal();
-      }
-      else {
-        if (rowPointer > -1) {
-          beforeFirst();
-          row->installCursorAtPosition(rowPointer > -1 ? rowPointer : 0);
-          lastRowPointer= -1;
-        }
-        growDataArray(true);
-        for (std::size_t rowNum= 0; rowNum < dataSize; ++rowNum) {
-          row->fetchNext();
-          row->cacheCurrentRow(data[rowNum], columnInformationLength);
-        }
-        for (auto& colInfo : columnsInformation) {
-          colInfo->makeLocalCopy();
-        }
-        //columnNameMap.init(columnsInformation);
-        rowPointer= preservedPosition;
-        fetchSize= 0;
-      }
-    }
     // else it is already cached in case of Text protocol
   }
+
 }
 }
 }
