@@ -117,6 +117,7 @@ namespace mariadb
     OPTIONS_FIELD(defaultFetchSize),
     OPTIONS_FIELD(tlsPeerFPList),
     OPTIONS_FIELD(log),
+    OPTIONS_FIELD(logname),
     OPTIONS_FIELD(profileSql),
     OPTIONS_FIELD(maxQuerySizeToLog),
     OPTIONS_FIELD(slowQueryThresholdNanos),
@@ -140,6 +141,7 @@ namespace mariadb
     OPTIONS_FIELD(useReadAheadInput),
     OPTIONS_FIELD(serverRsaPublicKeyFile),
     OPTIONS_FIELD(tlsPeerFP),
+    OPTIONS_FIELD(initSql),
     OPTIONS_FIELD(restrictedAuth)
   };
 
@@ -179,6 +181,11 @@ namespace mariadb
             this->*fieldPtr= cit->second.defaultValue;
             break;
           }
+          case ClassField<Options>::VUINT32: {
+            uint32_t Options::*fieldPtr= it.second;
+            this->*fieldPtr= static_cast<uint32_t>(cit->second.defaultValue);
+            break;
+          }
           case ClassField<Options>::VINT64: {
             int64_t Options::*fieldPtr= it.second;
             this->*fieldPtr= cit->second.defaultValue;
@@ -237,7 +244,7 @@ namespace mariadb
 
   bool Options::equals(Options* opt)
   {
-    if (opt == NULL) {
+    if (opt == nullptr) {
       return false;
     }
     if (this == opt) {
@@ -520,6 +527,9 @@ namespace mariadb
     if (!(tlsPeerFPList.compare(opt->tlsPeerFPList) == 0)) {
       return false;
     }
+    if (!(initSql.compare(opt->initSql) == 0)) {
+      return false;
+    }
     if (!(restrictedAuth.compare(opt->restrictedAuth) == 0)) {
       return false;
     }
@@ -619,6 +629,7 @@ namespace mariadb
     result= 31*result + (!nonMappedOptions.empty() ? hashProps(nonMappedOptions) : 0);
 
     result= 31*result + (!tlsPeerFPList.empty() ? tlsPeerFPList.hashCode() : 0);
+    result= 31*result + (!initSql.empty() ? initSql.hashCode() : 0);
     result= 31*result + (!restrictedAuth.empty() ? restrictedAuth.hashCode() : 0);
 
     return result;
