@@ -606,8 +606,9 @@ void bugs::bug66871()
   con->close();
   con.reset();
   
-  ASSERT_EQUALS(res->getInt(1), 1);
-  ASSERT(!res->next());
+  // This is not promised any more. But should still work, if resultset was not streamed. With streamed these will throw
+  // ASSERT_EQUALS(res->getInt(1), 1);
+  // ASSERT(!res->next());
 
   try
   {
@@ -1075,18 +1076,21 @@ void bugs::bug21067193()
 //	Error....
       throw e;
   }
-
-
 }
 
+
 void bugs::bug21152054()
 {
-
   stmt->execute("DROP TABLE IF EXISTS bug21152054");
   stmt->execute("create table bug21152054(c1 int);" );
   stmt->execute("insert into  bug21152054 values(1), (2), (3), (4);" );
   pstmt.reset( con->prepareStatement("select c1 from bug21152054;") );
   res.reset( pstmt->executeQuery() );
+
+  if (res->getType() == sql::ResultSet::TYPE_FORWARD_ONLY)
+  {
+    SKIP("Test doesn't make sense for ResultSet::TYPE_FORWARD_ONLY resultsets");
+  }
   ASSERT_EQUALS(true, res->absolute(4));
   ASSERT_EQUALS(4, res->getInt(1));
   int line = 4;
@@ -1106,10 +1110,10 @@ void bugs::bug22292073()
     SKIP("Server does not support tested functionality(JSON type)")
   }
   stmt->execute("DROP TABLE IF EXISTS bug22292073");
-  stmt->execute("create table bug22292073 (jdoc JSON);");
-  stmt->execute("insert into bug22292073 values('{ \"name\": \"abc\", \"age\": 1 , \"misc\":\
+  stmt->execute("CREATE TABLE bug22292073 (jdoc JSON);");
+  stmt->execute("INSERT INTO bug22292073 VALUES('{ \"name\": \"abc\", \"age\": 1 , \"misc\":\
                 1.2}'), ('{ \"name\": \"abcdef\", \"age\": 31 , \"misc\": 1.237843}');");
-  pstmt.reset(con->prepareStatement("select JSON_EXTRACT(jdoc, '$.age') from bug22292073;"));
+  pstmt.reset(con->prepareStatement("SELECT JSON_EXTRACT(jdoc, '$.age') FROM bug22292073;"));
   res.reset(pstmt->executeQuery());
 
   res->next();
@@ -1169,7 +1173,14 @@ void bugs::bug23212333()
 
   pstmt.reset(con->prepareStatement("SELECT id FROM bug23212333"));
   res.reset(pstmt->executeQuery());
-  ASSERT(res->relative(1));
+  if (res->getType() == sql::ResultSet::TYPE_FORWARD_ONLY)
+  {
+    ASSERT(res->next());
+  }
+  else
+  {
+    ASSERT(res->relative(1));
+  }
   sql::SQLString str(res->getString(1));
   ASSERT_EQUALS(charCount, static_cast<uint64_t>(str.length()));
 }
@@ -1256,7 +1267,7 @@ void bugs::concpp48()
   }
   catch (sql::SQLException & e)
   {
-    ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+    ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
   }
   try
   {
@@ -1265,7 +1276,7 @@ void bugs::concpp48()
   }
   catch (sql::SQLException & e)
   {
-    ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+    ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
   }
   try
   {
@@ -1274,7 +1285,7 @@ void bugs::concpp48()
   }
   catch (sql::SQLException & e)
   {
-    ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+    ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
   }
   try
   {
@@ -1283,7 +1294,7 @@ void bugs::concpp48()
   }
   catch (sql::SQLException & e)
   {
-    ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+    ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
   }
   try
   {
@@ -1292,7 +1303,7 @@ void bugs::concpp48()
   }
   catch (sql::SQLException & e)
   {
-    ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+    ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
   }
   try
   {
@@ -1301,7 +1312,7 @@ void bugs::concpp48()
   }
   catch (sql::SQLException & e)
   {
-    ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+    ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
   }
   try
   {
@@ -1310,7 +1321,7 @@ void bugs::concpp48()
   }
   catch (sql::SQLException & e)
   {
-    ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+    ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
   }
   try
   {
@@ -1319,7 +1330,7 @@ void bugs::concpp48()
   }
   catch (sql::SQLException & e)
   {
-    ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+    ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
   }
   try
   {
@@ -1328,7 +1339,7 @@ void bugs::concpp48()
   }
   catch (sql::SQLException & e)
   {
-    ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+    ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
   }
   try
   {
@@ -1337,7 +1348,7 @@ void bugs::concpp48()
   }
   catch (sql::SQLException & e)
   {
-    ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+    ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
   }
   try
   {
@@ -1346,7 +1357,7 @@ void bugs::concpp48()
   }
   catch (sql::SQLException & e)
   {
-    ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+    ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
   }
   try
   {
@@ -1355,7 +1366,7 @@ void bugs::concpp48()
   }
   catch (sql::SQLException & e)
   {
-    ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+    ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
   }
   try
   {
@@ -1364,7 +1375,7 @@ void bugs::concpp48()
   }
   catch (sql::SQLException & e)
   {
-    ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+    ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
   }
   try
   {
@@ -1373,7 +1384,7 @@ void bugs::concpp48()
   }
   catch (sql::SQLException & e)
   {
-    ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+    ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
   }
 
   /////////// Now all the same with CallableStatement ///////////
@@ -1392,7 +1403,7 @@ void bugs::concpp48()
     }
     catch (sql::SQLException & e)
     {
-      ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+      ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
     }
     try
     {
@@ -1401,7 +1412,7 @@ void bugs::concpp48()
     }
     catch (sql::SQLException & e)
     {
-      ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+      ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
     }
     try
     {
@@ -1410,7 +1421,7 @@ void bugs::concpp48()
     }
     catch (sql::SQLException & e)
     {
-      ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+      ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
     }
     try
     {
@@ -1419,7 +1430,7 @@ void bugs::concpp48()
     }
     catch (sql::SQLException & e)
     {
-      ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+      ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
     }
     try
     {
@@ -1428,7 +1439,7 @@ void bugs::concpp48()
     }
     catch (sql::SQLException & e)
     {
-      ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+      ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
     }
     try
     {
@@ -1437,7 +1448,7 @@ void bugs::concpp48()
     }
     catch (sql::SQLException & e)
     {
-      ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+      ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
     }
     try
     {
@@ -1446,7 +1457,7 @@ void bugs::concpp48()
     }
     catch (sql::SQLException & e)
     {
-      ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+      ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
     }
     try
     {
@@ -1455,7 +1466,7 @@ void bugs::concpp48()
     }
     catch (sql::SQLException & e)
     {
-      ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+      ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
     }
     try
     {
@@ -1464,7 +1475,7 @@ void bugs::concpp48()
     }
     catch (sql::SQLException & e)
     {
-      ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+      ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
     }
     try
     {
@@ -1473,7 +1484,7 @@ void bugs::concpp48()
     }
     catch (sql::SQLException & e)
     {
-      ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+      ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
     }
     try
     {
@@ -1482,7 +1493,7 @@ void bugs::concpp48()
     }
     catch (sql::SQLException & e)
     {
-      ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+      ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
     }
     try
     {
@@ -1491,7 +1502,7 @@ void bugs::concpp48()
     }
     catch (sql::SQLException & e)
     {
-      ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+      ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
     }
     try
     {
@@ -1500,7 +1511,7 @@ void bugs::concpp48()
     }
     catch (sql::SQLException & e)
     {
-      ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+      ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
     }
     try
     {
@@ -1509,7 +1520,7 @@ void bugs::concpp48()
     }
     catch (sql::SQLException & e)
     {
-      ASSERT(e.getMessage().find_first_of("cannot be called on PreparedStatement") != std::string::npos);
+      ASSERT(e.getMessage().find("cannot be called on PreparedStatement") != sql::SQLString::npos);
     }
   }
   catch (sql::SQLException &)
