@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2008, 2018, Oracle and/or its affiliates. All rights reserved.
- *               2020, 2023 MariaDB Corporation AB
+ *               2020, 2025 MariaDB Corporation plc
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
@@ -340,7 +340,7 @@ void unit_fixture::setUp()
     }
     catch (sql::SQLException& sqle)
     {
-      logErr(String("Couldn't get connection") + sqle.what());
+      logErr(String("Couldn't get connection: ") + sqle.what());
       logDebug("Host:" + url + ", UID:" + user + ", Schema:" + db + ", Tls:" + (useTls ? "yes" : "no")/* + ", Pwd:" + passwd + "<<<"*/);
       for (auto& cit : commonProperties) {
         String property(cit.first.c_str());
@@ -393,7 +393,12 @@ void unit_fixture::tearDown()
     }
     else {
       // resetting the connection for future use
-      con->reset();
+      try {
+        con->reset();
+      }
+      catch (sql::SQLException&) {
+        con.reset();
+      }
     }
   }
 }
@@ -759,7 +764,7 @@ bool unit_fixture::isSkySqlHA() const
 bool unit_fixture::isMaxScale() const
 {
   static bool MaxScaleOnTravis= (std::getenv("MAXSCALE_TEST_DISABLE") != nullptr ||
-    std::getenv("srv") != nullptr && strcmp(std::getenv("srv"), "maxscale") == 0);
+    (std::getenv("srv") != nullptr && strcmp(std::getenv("srv"), "maxscale") == 0));
   return MaxScaleOnTravis;
 }
 
